/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __reExport = (target, mod, secondTarget) => (__copyProps(target, mod, "default"), secondTarget && __copyProps(secondTarget, mod, "default"));
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/Downloader.ts
var Downloader_exports = {};
__export(Downloader_exports, {
  ImageRegistryFetcher: () => ImageRegistryFetcher,
  URLDownloader: () => URLDownloader,
  b64DecodeUnicode: () => b64DecodeUnicode,
  default: () => Downloader
});
module.exports = __toCommonJS(Downloader_exports);

// esm-externals:@splunk/ui-utils/i18n
var i18n_exports = {};
__export(i18n_exports, {
  default: () => i18n_default
});
var defaultImport = __toESM(require("@splunk/ui-utils/i18n"));
__reExport(i18n_exports, require("@splunk/ui-utils/i18n"));
var i18n_default = "default" in defaultImport ? defaultImport.default : defaultImport;

// src/Downloader.ts
var b64DecodeUnicode = (base64Data) => {
  return decodeURIComponent(
    atob(base64Data).split("").map((c) => {
      return `%${`00${c.charCodeAt(0).toString(16)}`.slice(-2)}`;
    }).join("")
  );
};
var ImageRegistryFetcher = class {
  constructor(context) {
    this.context = context;
  }
  downloadWithProgress(url, complete, progress, error) {
    return __async(this, null, function* () {
      const imageRegistry = this.context;
      if (!imageRegistry) {
        error((0, i18n_exports._)("Missing Image Context"));
      }
      try {
        const imageData = yield imageRegistry.getByURL(url);
        const [mime, svgBase64] = imageData.dataURI.split("base64,");
        if (mime && mime.toLowerCase().indexOf("image/svg+xml") > -1) {
          complete(b64DecodeUnicode(svgBase64));
        } else {
          error((0, i18n_exports._)(`Unsupported MIME type ${mime} for url ${url}`));
        }
      } catch (e) {
        error((0, i18n_exports._)(`Error retrieving ${url} from Image Registry, error was '${e.message}'`), e);
      }
    });
  }
};
var URLDownloader = class {
  downloadWithProgress(url, complete, progress, error) {
    fetch(url).then(
      (response) => __async(this, null, function* () {
        if (!response.ok) {
          const msg = `Fetch not Ok for ${url}, status was '${response.statusText}'`;
          console.warn(msg);
          error(msg);
          return;
        }
        const reader = response.body.getReader();
        const contentLength = +response.headers.get("Content-Length");
        let bytesDownloaded = 0;
        const chunks = [];
        let done = false;
        while (!done) {
          const res = yield reader.read();
          done = res.done;
          const { value } = res;
          if (value) {
            chunks.push(value);
            bytesDownloaded += value.length;
            progress({
              // call the progress callback...can enable the UI to show progress
              bytesDownloaded,
              contentLength,
              message: `Received ${bytesDownloaded} bytes of ${contentLength || "unknown size"}`
            });
          }
        }
        const chunksAll = new Uint8Array(bytesDownloaded);
        let position = 0;
        for (const chunk of chunks) {
          chunksAll.set(chunk, position);
          position += chunk.length;
        }
        progress({
          bytesDownloaded,
          contentLength,
          message: "decoding chunks"
        });
        const svg = new TextDecoder("utf-8").decode(chunksAll);
        complete(svg);
      })
    ).catch((e) => {
      const stringifiedError = e.toString();
      console.error(stringifiedError);
      if (stringifiedError.includes("Failed to fetch")) {
        error(
          `Cannot fetch the image "${url}" due to a cross-origin resource sharing (CORS) policy`,
          e
        );
      } else {
        error(`Fetch error for '${url}', error was '${e.toString()}'`, e);
      }
    });
  }
};
var Downloader = class {
  constructor(downloaders) {
    this.downloaders = {};
    this.downloaders = downloaders;
  }
  downloadWithProgress(url, complete, progress, error) {
    let protocol = "";
    let isUrl = true;
    try {
      protocol = new URL(url).protocol;
    } catch (e) {
      isUrl = false;
    }
    if (isUrl) {
      try {
        const [type = ""] = protocol.split(":");
        if (this.downloaders[type]) {
          this.downloaders[type].downloadWithProgress(url, complete, progress, error);
        } else {
          error(`Unsupported URL protocol '${type}' for SVG download: ${url}`);
        }
      } catch (e) {
        error(`Unexpected error downloading SVG: ${e.message}`);
      }
    } else {
      complete(url);
    }
  }
};
//# sourceMappingURL=Downloader.js.map
