/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/trellisUtils.ts
var trellisUtils_exports = {};
__export(trellisUtils_exports, {
  CHART: () => CHART,
  STATS: () => STATS,
  TIMECHART: () => TIMECHART,
  UNKNOWN: () => UNKNOWN,
  checkForTrellisFieldMetaInfo: () => checkForTrellisFieldMetaInfo,
  extractAllAggregationKeysChart: () => extractAllAggregationKeysChart,
  extractDataSourceAggregationKeysChart: () => extractDataSourceAggregationKeysChart,
  extractDataSourceAggregationKeysStats: () => extractDataSourceAggregationKeysStats,
  extractSplitByFieldsChartFields: () => extractSplitByFieldsChartFields,
  extractSplitByFieldsForStats: () => extractSplitByFieldsForStats,
  extractSplitByFieldsForUnknown: () => extractSplitByFieldsForUnknown,
  getAllPossibleSplitByFields: () => getAllPossibleSplitByFields,
  getTrellisDataSplitterMode: () => getTrellisDataSplitterMode,
  isChartCmdData: () => isChartCmdData,
  isStatsCmdData: () => isStatsCmdData,
  isTimeChartCmdData: () => isTimeChartCmdData
});
module.exports = __toCommonJS(trellisUtils_exports);
var CHART = "chart";
var TIMECHART = "timechart";
var STATS = "stats";
var UNKNOWN = "unknown";
var getTrellisDataSplitterMode = (fields) => {
  if (!fields) {
    return UNKNOWN;
  }
  if (isTimeChartCmdData(fields)) {
    return TIMECHART;
  }
  if (isChartCmdData(fields)) {
    return CHART;
  }
  if (isStatsCmdData(fields)) {
    return STATS;
  }
  return UNKNOWN;
};
var isTimeChartCmdData = (fields) => {
  return fields && fields.length > 0 && fields[0] && fields[0].name === "_time" && fields[0].groupby_rank === "0";
};
var isChartCmdData = (fields) => {
  if (!fields || !fields.length) {
    return false;
  }
  if (!checkForTrellisFieldMetaInfo(fields)) {
    return false;
  }
  if (fields.length > 1) {
    return fields[0].name !== "_time" && fields[0].groupby_rank === "0" && !fields[1].groupby_rank;
  }
  for (let i = 1; i < fields.length; i += 1) {
    if (!fields[i].splitby_field) {
      return false;
    }
  }
  return true;
};
var isStatsCmdData = (fields) => {
  if (!fields || !fields.length) {
    return false;
  }
  if (!checkForTrellisFieldMetaInfo(fields)) {
    return false;
  }
  const groupByRanks = [];
  for (let i = 0; i < fields.length; i += 1) {
    if (fields[i].splitby_field) {
      return false;
    }
    if (fields[i].groupby_rank) {
      groupByRanks.push(fields[i].groupby_rank);
    }
  }
  return groupByRanks.length > 0 && groupByRanks.length < fields.length;
};
var checkForTrellisFieldMetaInfo = (fields) => {
  if (!fields || !fields.length) {
    return false;
  }
  for (let i = 0; i < fields.length; i += 1) {
    if (fields[i].groupby_rank || fields[i].splitby_field || fields[i].splitby_value || fields[i].data_source) {
      return true;
    }
  }
  return false;
};
var extractSplitByFieldsChartFields = (fields) => {
  const fieldsWithoutTime = fields.filter(({ name }) => name !== "_time");
  const splitByFields = [];
  fieldsWithoutTime.forEach((field) => {
    if (field.splitby_field) {
      splitByFields.push(field.splitby_field);
    }
  });
  return isTimeChartCmdData(fields) ? [...new Set(splitByFields)] : [fields[0].name, ...new Set(splitByFields)];
};
var extractDataSourceAggregationKeysChart = (fields) => {
  const fieldsWithoutTime = fields.filter(({ name }) => name !== "_time");
  const aggregations = [];
  fieldsWithoutTime.forEach((field) => {
    if (field.data_source) {
      aggregations.push(field.data_source);
    }
  });
  return [...new Set(aggregations)];
};
var extractAllAggregationKeysChart = (fields) => {
  let aggregations = [];
  aggregations = extractDataSourceAggregationKeysChart(fields);
  if (aggregations && aggregations.length) {
    return aggregations;
  }
  if ((!aggregations || !aggregations.length) && fields.length > 1) {
    for (let i = 1; i < fields.length; i += 1) {
      const fieldName = fields[i].name;
      if (!fieldName.startsWith("_")) {
        aggregations.push(fieldName);
      }
    }
  }
  return aggregations;
};
var extractDataSourceAggregationKeysStats = (fields) => {
  const aggregations = [];
  fields.forEach((field) => {
    if (field.groupby_rank === void 0) {
      aggregations.push(field.name);
    }
  });
  return aggregations;
};
var extractSplitByFieldsForStats = (fields) => {
  if (!fields) {
    return [];
  }
  const fieldsWithGroupByRank = fields.filter((field) => field.groupby_rank);
  return fieldsWithGroupByRank.map((field) => field.name);
};
var extractSplitByFieldsForUnknown = (fields) => {
  if (!fields) {
    return [];
  }
  return fields.filter((field) => !(field == null ? void 0 : field.name.startsWith("_"))).map((field) => field.name);
};
var getAllPossibleSplitByFields = (fields) => {
  const result = { fields: [], aggregations: [] };
  if (!fields || !fields.length) {
    return result;
  }
  const splitMode = getTrellisDataSplitterMode(fields);
  if (splitMode === TIMECHART || splitMode === CHART) {
    return {
      fields: extractSplitByFieldsChartFields(fields),
      aggregations: extractAllAggregationKeysChart(fields)
    };
  }
  if (splitMode === STATS) {
    return {
      fields: [...extractSplitByFieldsForStats(fields)],
      aggregations: [...extractDataSourceAggregationKeysStats(fields)]
    };
  }
  if (splitMode === UNKNOWN) {
    const fieldNames = extractSplitByFieldsForUnknown(fields);
    return fieldNames.length === 1 ? {
      fields: [],
      aggregations: [...fieldNames]
      // only populate the aggregation
    } : {
      fields: [...fieldNames],
      aggregations: [...fieldNames]
    };
  }
  return result;
};
//# sourceMappingURL=trellisUtils.js.map
